using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;

namespace WiFiPad
{
    /// <summary>
    /// This is the main type for your app
    /// </summary>
    public class CarGUI : Microsoft.Xna.Framework.Game
    {
        GraphicsDeviceManager graphics;
        SpriteBatch spriteBatch;
        Texture2D  background, controls, vumeter, blank;
		SpriteFont font;
		
		CarControl car = new CarControl();
		bool ObstaclesStop = true;
		string Host = "192.168.1.37";
		int Port = 2000;
		double lastGameTime =0;
		double lastSonarGameTime = 0;
		GamePadState previusState;
		float direccio =0;
		string sonar ="";		
		float speed=0;

		Vector2 aButtonPos = new Vector2(700, 450);
		Vector2 bButtonPos = new Vector2(780, 450);
		Vector2 stickPos = new Vector2(300-68/2, 300-68/2);
		Rectangle aButtonRect = new Rectangle(0, 90, 60,60);
		Rectangle bButtonRect = new Rectangle(0, 160, 60, 60);
		Rectangle thumbStickRect = new Rectangle(0, 0, 80,80);
		int vumeterLength = 200;
		int vumeterWidth  = 76;
		int vumeterPosX   = 550;
		int vumeterPosY   = 310;

        /// <summary>
        /// Constructor
        /// </summary>
        public CarGUI()
        {
            graphics = new GraphicsDeviceManager(this);
            Content.RootDirectory = "Content";
			
#if !WINDOWS
			graphics.IsFullScreen = true;					
#else
             graphics.PreferredBackBufferHeight = 768;
            graphics.PreferredBackBufferWidth = 1024;
#endif
        }

        /// <summary>
        /// Allows the game to perform any initialization it needs to before starting to run.
        /// This is where it can query for any required services and load any non-graphic
        /// related content.  Calling base.Initialize will enumerate through any components
        /// and initialize them as well.
        /// </summary>
        protected override void Initialize()
        {
			base.Initialize();
        }

        /// <summary>
        /// LoadContent will be called once per game and is the place to load
        /// all of your content.
        /// </summary>
        protected override void LoadContent()
        {
            // Create a new SpriteBatch, which can be used to draw textures.
            spriteBatch = new SpriteBatch(GraphicsDevice);

            background = Content.Load<Texture2D>("back"); 
            controls = Content.Load<Texture2D>("botons");
            vumeter = Content.Load<Texture2D>("vumeter_empty");
            blank = Content.Load<Texture2D>("blank");
			font = Content.Load<SpriteFont>("font");
            
#if !WINDOWS
            // Set the virtual GamePad
            ButtonDefinition AButton = new ButtonDefinition();
            AButton.Texture = controls;
			AButton.Position = aButtonPos; 
            AButton.Type = Buttons.A;
            AButton.TextureRect = aButtonRect;

            ButtonDefinition BButton = new ButtonDefinition();
            BButton.Texture = controls;
			BButton.Position = bButtonPos; 
            BButton.Type = Buttons.B;
            BButton.TextureRect = bButtonRect;

            GamePad.ButtonsDefinitions.Add(BButton);
            GamePad.ButtonsDefinitions.Add(AButton);
			
            ThumbStickDefinition thumbStick = new ThumbStickDefinition();
			thumbStick.Position = stickPos; 
            thumbStick.Texture = controls;
            thumbStick.TextureRect = thumbStickRect;
			
            GamePad.LeftThumbStickDefinition = thumbStick;
            GamePad.Radius =200;
#endif

        }
		
        /// <summary>
        /// Allows the game to run logic such as updating the world,
        /// checking for collisions, gathering input, and playing audio.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Update(GameTime gameTime)
        {
			if (!car.Connected)
            {
                car.Init(Host, Port);
            }

			// Allows the game to exit
            if (GamePad.GetState(PlayerIndex.One).Buttons.Back == ButtonState.Pressed)
               this.Exit();

			if (GamePad.GetState(PlayerIndex.One).Buttons.A == ButtonState.Pressed 
			    && previusState.Buttons.A == ButtonState.Released)
			{
				car.Stop();
				sonar = "Sonar: "+ car.Sonar().ToString("0.00")+ " cm";
			}
						
			if (GamePad.GetState(PlayerIndex.One).Buttons.B == ButtonState.Pressed
			    && previusState.Buttons.B == ButtonState.Released)
			{
				car.Obstacles();
				ObstaclesStop = !ObstaclesStop;
			}
						
			// change the caracter position using thumbstick
			GamePadState gamepastatus = GamePad.GetState(PlayerIndex.One);

			speed = gamepastatus.ThumbSticks.Left.Y * 15f; 
			direccio = gamepastatus.ThumbSticks.Left.X * 35f;

			if((gameTime.TotalGameTime.TotalMilliseconds - lastGameTime)>500)
			{
				lastGameTime = gameTime.TotalGameTime.TotalMilliseconds;
                car.Traction((int)speed);
                car.Direcction((int)direccio); 
			}
			if((gameTime.TotalGameTime.TotalMilliseconds - lastSonarGameTime)>2500)
			{
				lastSonarGameTime = gameTime.TotalGameTime.TotalMilliseconds;
				sonar = "Sonar: "+ car.Sonar().ToString("0.00")+ " cm";
				car.Traction((int)speed);
				car.Direcction((int)direccio); 
			}

			previusState = GamePad.GetState(PlayerIndex.One);
            base.Update(gameTime);
        }

        
        /// <summary>
        /// This is called when the game should draw itself.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Draw(GameTime gameTime)
        {
			graphics.GraphicsDevice.Clear(Color.GreenYellow); 
			spriteBatch.Begin();

            spriteBatch.Draw(background, new Vector2(100,100), null, Color.White, 0f,
			                 Vector2.Zero, 1f, SpriteEffects.None, 0);

            // Draw the virtual GamePad
#if !WINDOWS
			GamePad.Draw(gameTime,spriteBatch);
#else
			spriteBatch.Draw(controls,stickPos, thumbStickRect, Color.White);
			spriteBatch.Draw(controls, aButtonPos, aButtonRect, Color.White);
			spriteBatch.Draw(controls, bButtonPos, bButtonRect, Color.White);
#endif

			spriteBatch.Draw(blank, new Rectangle(vumeterPosX, vumeterPosY-vumeterLength, vumeterWidth, vumeterLength), Color.YellowGreen);
			spriteBatch.DrawString(font, speed.ToString("0"), new Vector2(vumeterPosX+30, vumeterPosY), Color.DarkGreen);
			spriteBatch.Draw(blank, new Rectangle(vumeterPosX, vumeterPosY+30 , vumeterWidth, vumeterLength), Color.YellowGreen);

			int d = (int)(previusState.ThumbSticks.Left.Y * vumeterLength);
			if (d != 0)
			{
                if (d > 0)
                {
                    spriteBatch.Draw(vumeter, new Vector2(vumeterPosX + vumeterWidth, vumeterPosY + vumeterLength - d),
                                     new Rectangle(0, vumeterLength - d, vumeterWidth, d),
                                     Color.White, 0, new Vector2(vumeterWidth, vumeterLength), 1f, SpriteEffects.None, 0);
                }
                else
                {
                    spriteBatch.Draw(vumeter, new Vector2(vumeterPosX, vumeterPosY + 30),
                                     new Rectangle(0, vumeterLength + d, vumeterWidth, -d),
                                     Color.White, 0, new Vector2(0, 0), 1f, SpriteEffects.FlipVertically, 0);
                }
			}

			// Show status
			spriteBatch.DrawString(font, sonar, 
			                       new Vector2(vumeterPosX+100, vumeterPosY- vumeterLength), Color.DarkGreen);
			spriteBatch.DrawString(font, "Detect obstacles: "+ ObstaclesStop, 
			                       new Vector2(vumeterPosX+100, vumeterPosY- vumeterLength+40), Color.DarkGreen);

            spriteBatch.End();
            base.Draw(gameTime);
        }
    }
}
