/*
 * arplab.c
 *
 *  Created on: 03/11/2012
 *      Author: Francec Pujol Contreras
 */

#include <stdio.h>
#include <string.h>
#include <stdarg.h>
#include <stdlib.h>

/* FreeRTOS.org includes. */
#include "FreeRTOS.h"
#include "task.h"
#include "semphr.h"
#include "type.h"

#include "uart.h"
#include "uartTools.h"
#include "wifly.h"
#include "arplab.h"

#define RESPONSE_BUFF_LEN 					( 512 )
static char response[RESPONSE_BUFF_LEN];

static uint32_t uart = 3;
static int32_t debugUart = -1;

/* Declare a variable of type xSemaphoreHandle.  This is used to reference the
mutex type semaphore that is used to ensure mutual exclusive access to stdout. */
static xSemaphoreHandle xMutex;


/*****************************************************************************
** Function name:		
**
** Descriptions:		Send ... value.
**
** parameters:			None
** Returned value:		None
**
*****************************************************************************/
uint32_t arpLabInit(uint32_t useUart, uint32_t baudrate, enumSecurity mode, char *wlan, char *wlanPassword)
{
	uint32_t ret = FALSE;
	if (useUart != debugUart && useUart <MAX_UART_NUMBER)
	{
		uart = useUart;

		wiFlyInit(uart, baudrate);
		ret = wiFlyOpen(response);
		debugMessage(response);

		if (ret == TRUE)
		{
			ret = wiFlyConnect(mode, wlan, wlanPassword, response);
			debugMessage(response);
		}

		if (ret == TRUE)
		{
			wiFlyClose(response);
			debugMessage(response);
			// Before a semaphore is used it must be explicitly created.
			xMutex = xSemaphoreCreateMutex();

			// Verify that the semaphore was created successfully.
			if( xMutex != NULL )
				ret = TRUE;
			else
				ret = FALSE;
		}
	}
	return ret;
}

/*****************************************************************************
** Function name:		
**
** Descriptions:		Send ... value.
**
** parameters:			None
** Returned value:		None
**
*****************************************************************************/
uint32_t arpLabShowDebug(int32_t useUart, uint32_t baudrate)
{
	uint32_t ret = FALSE;
	if (useUart != uart && useUart < MAX_UART_NUMBER)
	{
		debugUart = useUart;
		if (debugUart != -1)
		{
			// init uart
			//UARTInit(debugUart, baudrate);
			printfUartInit(debugUart, baudrate);
		}
		ret = TRUE;
	}
	return ret;
}

/*****************************************************************************
** Function name:		arpLabGetNumber
**
** Descriptions:		Send requested to Arp@lab rest service for
** 						a integer value.
**
** parameters:			uart number
** Returned value:		Arp@Lab number
**
*****************************************************************************/
uint32_t arpLabGetNumber()
{
	uint32_t number =0;
	if (xMutex != NULL)
	{
		/* The semaphore is created before the scheduler is started so already
		exists by the time this task executes.

		Attempt to take the semaphore, blocking indefinitely if the mutex is not
		available immediately.  The call to xSemaphoreTake() will only return when
		the semaphore has been successfully obtained so there is no need to check the
		return value.  If any other delay period was used then the code must check
		that xSemaphoreTake() returns pdTRUE before accessing the resource (in this
		case standard out. */
		xSemaphoreTake( xMutex, portMAX_DELAY );
		{
			wiFlyOpen(response);
			debugMessage(response);

			number = wiFlyWebGet("http://arpalab.appspot.com/rest/feed", response);
			debugMessage(response);

			wiFlyClose(response);
			debugMessage(response);
		}
		xSemaphoreGive( xMutex );
	}
	return number;
}

/*****************************************************************************
** Function name:		
**
** Descriptions:		Send ... value.
**
** parameters:			None
** Returned value:		None
**
*****************************************************************************/
void arpLabSendValue(uint32_t value, char *nid, char *ip, char *app)
{
	static char buffer[256];

	if (xMutex != NULL)
	{
		xSemaphoreTake( xMutex, portMAX_DELAY );
		{

			wiFlyOpen(response);
			debugMessage(response);

			sprintf(buffer,"http://arpalab.appspot.com/rest/stats?nid=%s&ip=%s&app=%s&data=%i", nid, ip, app, value);
			debugMessage(response);

			wiFlyWebSend(buffer, response);
			debugMessage(response);

			wiFlyClose(response);
			debugMessage(response);
		}
		xSemaphoreGive( xMutex );
	}
}


/*****************************************************************************
** Function name:		
**
** Descriptions:		Send ... value.
**
** parameters:			None
** Returned value:		None
**
*****************************************************************************/
void debugMessage(char *message)
{
	if (debugUart != UART_NONE){
		uartPrintfLn(debugUart, response);
	}
}
